#
# Licensed Materials - Property of IBM
# 5765-FS1
# (c) Copyright IBM Corp. 2003  All Rights Reserved
# US Government Users Restricted Rights - Use, duplication or 
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
# Linux Data Structures and Functions
#
#  This file contains the data structures and functions needed by the 
#  Storage Tank one button data collection srcipt (OBDC), which is pmf.pl.
#  This file is included into the pmf.pl script if it is run on an Linux
#  machine.  See the pmf.pl script for more information about this file.
#

package OBDC_Linux;

use strict;
use OBDC_Common;

my ($smisHome, $scriptOutput, $archiveFile, $serverOutputDir, $smisCmdWaitTime);

our (%Pmf_HW_NW_LAN_Funcs, %Pmf_HW_Proc_Funcs, 
     %Pmf_HW_Mem_Funcs, %Pmf_HW_Storage_Funcs, %Pmf_HW_Storage_Subcomps, 
     %Pmf_HW_NW_Subcomps, %Pmf_HW); 
our (%Pmf_SW_SANFS_Server_Funcs, 
     %Pmf_SW_SANFS_Subcomps, %Pmf_SW_Storage_Funcs, 
     %Pmf_SW_NW_LAN_Funcs, %Pmf_SW_OS_Proc_Funcs, 
     %Pmf_SW_OS_Funcs, %Pmf_SW_NW_Subcomps, %Pmf_SW_OS_Subcomps, %Pmf_SW,
     %Pmf_SW_OS_SW_Funcs);

#
# Commands
#
# The commands are split into two groups.  The safe commands
# provide generally useful data and have no bad side effects on the clients
# or servers.  The internal show commands are only executed with the 
# 'internal' collection option.  The internal commands either produce data
# that is only deemed appropriate for a limited set of problems, or have
# possible side effects on the clients which are not acceptable when 
# gathering data for most problems.
#

$smisCmdWaitTime = 30;  # Seconds to wait for a command to complete.

#
# Linux Hardware
#

%Pmf_HW_NW_LAN_Funcs = (
			"info" => {colopt => $OBDC_Common::collectBasic |
			 	             $OBDC_Common::collectFull,
				   func => \&Pmf_HW_NW_LANInfo},
			"status" => {colopt => $OBDC_Common::collectFull,
				     func => \&Pmf_HW_NW_LANStatus}
			);


%Pmf_HW_Proc_Funcs = (
		      "info" => {colopt => $OBDC_Common::collectBasic |
				           $OBDC_Common::collectFull,
				 func => \&Pmf_HW_ProcInfo}
		      );

%Pmf_HW_Mem_Funcs = (
		     "info" => {colopt => $OBDC_Common::collectBasic |
			                  $OBDC_Common::collectFull,
			         func => \&Pmf_HW_MemInfo}
		     );

%Pmf_HW_Storage_Funcs = (
			 "info" => {colopt => $OBDC_Common::collectBasic |
			                      $OBDC_Common::collectFull,
			            func => \&Pmf_HW_StorageInfo}
			 );

%Pmf_HW_Storage_Subcomps = (
			    "Local Disk" => {colopt => $OBDC_Common::collectAny,
					     roleopt => $OBDC_Common::roleAny,
					     functions => undef,
					     subcomps => undef},
			    "SAN Disk" => {colopt => $OBDC_Common::collectAny,
					   roleopt => $OBDC_Common::roleServer |
			                               $OBDC_Common::roleClient, 
					   functions => undef,
					   subcomps => undef},
			 );


%Pmf_HW_NW_Subcomps = (
		       "LAN" => {colopt => $OBDC_Common::collectAny,
				 roleopt => $OBDC_Common::roleAny,
				 functions => \%Pmf_HW_NW_LAN_Funcs,
				 subcomps => undef},
			 );

%Pmf_HW = (
	   "Processor" => {colopt => $OBDC_Common::collectAny,
			   roleopt => $OBDC_Common::roleAny,
			   functions => \%Pmf_HW_Proc_Funcs,
			   subcomps => undef},
	   "Memory" => {colopt => $OBDC_Common::collectAny,
			roleopt => $OBDC_Common::roleAny,
			functions => \%Pmf_HW_Mem_Funcs,
			subcomps => undef},
	   "Network" => {colopt => $OBDC_Common::collectAny,
			 roleopt => $OBDC_Common::roleAny,
			 functions => undef,
			 subcomps => \%Pmf_HW_NW_Subcomps},
	   "Storage" => {colopt => $OBDC_Common::collectAny,
			 roleopt => $OBDC_Common::roleAny,
			 functions => \%Pmf_HW_Storage_Funcs,
			 subcomps => \%Pmf_HW_Storage_Subcomps}
	   );

#
# Linux Hardware Functions
#
sub Pmf_HW_ProcInfo {
    &OBDC_Common::ExecuteShellCommands("cat /proc/cpuinfo");
}

sub Pmf_HW_MemInfo {
    &OBDC_Common::ExecuteShellCommands("cat /proc/meminfo");
}

sub Pmf_HW_NW_LANInfo {
    &OBDC_Common::ExecuteShellCommands("cat /proc/net/dev");
}

sub Pmf_HW_NW_LANStatus {
    &OBDC_Common::ExecuteShellCommands("ifconfig -a, netstat -nr");
}


sub Pmf_HW_StorageInfo {
    &OBDC_Common::ExecuteShellCommands("df -ahiT, cat /proc/partitions");
}

#
# Linux Software
#

%Pmf_SW_SANFS_Server_Funcs = (
			      "info" => {colopt => $OBDC_Common::collectBasic |
				                   $OBDC_Common::collectFull,
					 func => \&Pmf_SW_SANFSServerInfo},
			      "diags" => {colopt => $OBDC_Common::collectFull |
				                    $OBDC_Common::collectInternal,
					  func => \&Pmf_SW_SANFSServerDiags},
			      "logs" => {colopt => $OBDC_Common::collectFull,
					 func => \&Pmf_SW_SANFSServerLogs},
			      "perf" => {colopt => $OBDC_Common::collectPerf,
					 func => \&Pmf_SW_SANFSServerPerf},
			      );


%Pmf_SW_SANFS_Subcomps = (
			  "Server" => {colopt => $OBDC_Common::collectAny,
				       roleopt => $OBDC_Common::roleServer,
				       functions => \%Pmf_SW_SANFS_Server_Funcs,
				       subcomps => undef},
			 );

%Pmf_SW_Storage_Funcs = (
			 "info" => {colopt => $OBDC_Common::collectBasic |
				              $OBDC_Common::collectFull,
				       func => \&Pmf_SW_StorageInfo}
			);

%Pmf_SW_NW_LAN_Funcs = (
			"info" => {colopt => $OBDC_Common::collectBasic |
				             $OBDC_Common::collectFull,
				   func => \&Pmf_SW_NW_LAN_Info},
			"diags" => {colopt => $OBDC_Common::collectFull,
				    func => \&Pmf_SW_NW_LAN_Diags}
			);

%Pmf_SW_OS_Proc_Funcs = (
			 "info" => {colopt => $OBDC_Common::collectFull,
				    func => \&Pmf_SW_OS_Processes}
			 );

%Pmf_SW_OS_Funcs = (
		    "info" => {colopt => $OBDC_Common::collectBasic |
				         $OBDC_Common::collectFull,
			       func => \&Pmf_SW_OS_Info},
		    "logs" => {colopt => $OBDC_Common::collectFull,
			       func => \&Pmf_SW_OS_Logs}
		     );

%Pmf_SW_OS_SW_Funcs = (
		       "info" => {colopt => $OBDC_Common::collectFull,
				  func => \&Pmf_SW_OS_SW_Info}
		       );

%Pmf_SW_NW_Subcomps = (
		       "LAN" => {colopt => $OBDC_Common::collectAny,
				 roleopt => $OBDC_Common::roleAny,
				 functions => \%Pmf_SW_NW_LAN_Funcs,
				 subcomps => undef},
		       );


%Pmf_SW_OS_Subcomps = (
		       "Processes" => {colopt => $OBDC_Common::collectFull,
				       roleopt => $OBDC_Common::roleAny,
				       functions => \%Pmf_SW_OS_Proc_Funcs,
				       subcomps => undef},
		       "Software" => {colopt => $OBDC_Common::collectFull,
				      roleopt => $OBDC_Common::roleAny,
				      functions => \%Pmf_SW_OS_SW_Funcs,
				      subcomps => undef}
		       );

%Pmf_SW = (
	   "Operating System" => {colopt => $OBDC_Common::collectAny,
				  roleopt => $OBDC_Common::roleAny,
				  functions => \%Pmf_SW_OS_Funcs,
				  subcomps => \%Pmf_SW_OS_Subcomps},
	   "Network" => {colopt => $OBDC_Common::collectAny,
			 roleopt => $OBDC_Common::roleAny,
			 functions => undef,
			 subcomps => \%Pmf_SW_NW_Subcomps},

	   "Storage" => {colopt => $OBDC_Common::collectAny,
			 roleopt => $OBDC_Common::roleAny,
			 functions => \%Pmf_SW_Storage_Funcs,
			 subcomps => undef},
	   "SAN FS" => {colopt => $OBDC_Common::collectAny,
			roleopt => $OBDC_Common::roleAny,
			functions => undef,
			subcomps => \%Pmf_SW_SANFS_Subcomps}
	   );

#
# Linux Software Functions
#
sub Pmf_SW_OS_Info {
    &OBDC_Common::ExecuteShellCommands("uname -a, env");
    &OBDC_Common::ExecuteShellCommands("cat /proc/version, cat /proc/modules, cat /proc/locks");
}

sub Pmf_SW_OS_Processes {
    &OBDC_Common::ExecuteShellCommands("ps huxefww");
}

sub Pmf_SW_OS_Logs {
    &OBDC_Common::ExecuteShellCommands("cp -p /var/log/messages $scriptOutput");
    &OBDC_Common::ExecuteShellCommands("cp -p /var/log/warn $scriptOutput");
}

sub Pmf_SW_OS_SW_Info {
    &OBDC_Common::ExecuteShellCommands("rpm -qa");
}

sub Pmf_SW_NW_LAN_Info {
    &OBDC_Common::ExecuteShellCommands("netstat -a, cat /proc/net/arp");
}

sub Pmf_SW_NW_LAN_Diags{
    &OBDC_Common::ExecuteShellCommands("cat /proc/net/snmp");
}


sub Pmf_SW_StorageInfo {
    &OBDC_Common::ExecuteShellCommands("df -ahkT, cat /proc/mounts");
}

#
# Server Functions
#

sub Pmf_SW_SANFSServerInfo {
}

sub Pmf_SW_SANFSServerDiags {
    #
    # Find and copy over any core files.  Note that we only search the
    # server directory to avoid finding the files in the pmf directory.
    #
    # Exclude the files from _uninst directory since a couple of files 
    # elready has the word core in their names
    &OBDC_Common::ExecuteShellCommands("find $smisHome -path $smisHome/_uninst -prune -o -name \\*core\\* -print -exec cp -p {} $serverOutputDir \\;");

}

sub Pmf_SW_SANFSServerLogs {
    &OBDC_Common::ExecuteShellCommands("cp -p $smisHome/logs/\* $serverOutputDir");
    &OBDC_Common::ExecuteShellCommands("cp -p $smisHome/pegasus/logs/* $serverOutputDir");
    &OBDC_Common::ExecuteShellCommands("cp -p $smisHome/pegasus/cimserver.txt $serverOutputDir");
    &OBDC_Common::ExecuteShellCommands("cp -p $smisHome/pegasus/versions.prop $serverOutputDir");
    &OBDC_Common::ExecuteShellCommands("cp -p $smisHome/pegasus/cimserver_\*.conf $serverOutputDir");
}

sub Pmf_SW_SANFSServerPerf {
}


#
#  Pmf_Archive() --
#    Archive and compress all the data that was gathered
#
#    Arguments:
#        outDir -- the output directory that contains the files to archive
#                  Note: this is ignored because we already have this value from
#                        the initialization function.
#
sub Pmf_Archive {
    my ($tarFile, $tarDir);

    $tarDir  = $_[0];
    $tarFile = "${archiveFile}.tgz";
    system("cd $scriptOutput/..; tar cfz $tarFile $tarDir; rm -rf $tarDir");
}

sub Pmf_InitPlatform {
    my (%pmfData);

    %pmfData = &Pmf_Main::RetrievePmfData();
    $smisHome = $pmfData{"home"};
    $scriptOutput = $pmfData{"outputDir"};
    $serverOutputDir = $pmfData{"serverOutputDir"};
    $archiveFile = $pmfData{"archiveFile"};
}

1;
